#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   server_findJobFiles.py
#
#       jobIdからlogFileを探しだす
#
#   16/12/10    新規作成
#      12/17    getLogFileInJobFileを追加
#   17/07/12    実行時のcaseDirの表記を修正し、hostDirが取得できる様に修正。
#      07/21    getLogFileInJobFile:「#PJM」にも対応させる。
#      09/16    getCaseDir:バグ修正（folderが取得できなかった場合エラー停止）
#

import os
import sys
import time
import glob

#
#  findJobInCurrDir
#    currDir内で検索
def findJobInCurrDir(currDir, pattern):
    getFiles = []
    files = glob.glob(currDir+"/*")
    for nameDir in files:
        name = nameDir.split("/")[-1]
        if name.find(pattern) >= 0:
            dt = os.path.getmtime(nameDir)
            getFiles.append([dt, nameDir])
            if name == "jobId:"+name:
                break
    return getFiles

#
#  findJobFile
#    jobIdファイルを検索する
def findJobFile(wdir, findStr):
    getFiles = []
    flag = 0
    for (root, dirs, files) in os.walk(wdir):
        for name in files:
            if name.find(findStr) >= 0:
                fileName = root + "/" + name
                dt = os.path.getmtime(fileName)
                getFiles.append([dt, fileName])
                if name == "jobId:"+name:
                    flag = 1
                    break
        if flag == 1:
            break
    return getFiles

#
#  getCaseDir
#    filesのlistからcaseDirを探す
def getCaseDir(files):
    idFolder = ""
    for dtFile in files:
        dt = dtFile[0]
        nameDir = dtFile[1]
        caseName = nameDir.split("/")[-1]
        if caseName[:len("jobId:")] == "jobId:":
            bdt = time.time() - dt
            if bdt < 604800.0:
                idFolder = "/".join(nameDir.split("/")[:-1])
                break
    if idFolder == "":
        #file名にjobIdが含まれているfileを探す
        newFolders = []
        for dtFile in files:
            dt = dtFile[0]
            nameDir = dtFile[1]
            bdt = time.time() - dt
            newFolders.append([bdt, nameDir])
        if len(newFolders) > 0:
            newFolders.sort()
            if newFolders[0][0] < 604800.0:
                idFolder = newFolders[0][1]
    return idFolder

#
#  getLogFileInJobFile
#    jobFile内からlogFileを取得する
def getLogFileInJobFile(jobFile, pattern, jobMan):
    if jobMan == "#SBATCH":
        logFile = getLogFile_SBATCH(jobFile, pattern)
    elif jobMan == "#PJM":
        logFile = getLogFile_PJM(jobFile, pattern)
    return logFile

#  getLogFile_SBATCH
#    #SBATCHからlogFile名を取得
def getLogFile_SBATCH(jobFile, pattern):
    f=open(jobFile); lines=f.readlines(); f.close()
    logFile = ""
    for line in lines:
        words = line.split()
        if len(words) >= 3:
            #「#SBATCH -o」からlogFileを取得
            if words[0] == "#SBATCH" and words[1] == "-o":
                logFile = words[2]
                break
            #「# SBATCH -o」からlogFileを取得
            elif words[0] == "#" and words[1] == "SBATCH" and words[2] == "-o":
                if len(words) >= 4:
                    logFile = words[3]
                    break
    if logFile != "":
        #「%J」をjobIdに変換する
        if logFile.find("%J") >= 0:
            logFile = logFile.replace("%J", pattern)
        #logFileをfullPathに修正
        caseDir = "/".join(jobFile.split("/")[:-1])
        logFile = caseDir + "/" + logFile
    return logFile

#  getLogFile_PJM
#    #PJMからlogFile名を取得
def getLogFile_PJM(jobFile, pattern):
    f=open(jobFile); lines=f.readlines(); f.close()
    logFile = ""
    for line in lines:
        words = line.split()
        if len(words) >= 3:
            #「#PJM -o」からlogFileを取得
            if words[0] == "#PJM" and words[1] == "-o":
                logFile = words[2]
                break
            #「# PJM -o」からlogFileを取得
            elif words[0] == "#" and words[1] == "PJM" and words[2] == "-o":
                if len(words) >= 4:
                    logFile = words[3]
                    break
    if logFile != "":
        #logFileをfullPathに修正
        caseDir = "/".join(jobFile.split("/")[:-1])
        logFile = caseDir + "/" + logFile
    return logFile

#
#  getLogFile
#    logFileを取得
def getLogFile(caseDir, pattern, jobMan):
    #最初にjobId:ファイル内容を確認
    jobIdFile = caseDir + "/jobId:" + pattern
    if len(glob.glob(jobIdFile)) != 0:
        #1W以内のjobIdファイルを探す
        if (time.time() - os.path.getmtime(jobIdFile)) < 604800.0:
            #jobIdファイル内からjobFileを取得
            f=open(jobIdFile); lines=f.readlines(); f.close()
            if len(lines) >= 1:
                jobFile = caseDir + "/" + lines[0].split()[0]
                #jobFile内からlogFileを取得
                logFile = getLogFileInJobFile(jobFile, pattern, jobMan)
                if logFile != "":
                    return logFile
    #10分以内に作成されたfileからlogFileを探す
    newFiles = []
    files = glob.glob(caseDir + "/*")
    for name in files:
        if os.path.isfile(name) == True:
            dt = os.path.getmtime(name)
            bdt = time.time() - dt
            if bdt < 600.0:
                #10分以内のfile全てを取得
                newFiles.append([bdt, name])
    #取得した10分以内のfileからlogFileを探す
    newFiles.sort()
    logFile = ""
    for nameDir in newFiles:
        name = nameDir[1].split("/")[-1]
        if name == "solve.log":
            logFile = nameDir[1]
            break
        elif name.find("o"+pattern) >= 0:
            logFile = nameDir[1]
            break
        elif name.find(pattern+"o") >= 0:
            logFile = nameDir[1]
            break
        elif name.find("out") >= 0:
            logFile = nameDir[1]
            break
    return logFile


#メイン
if __name__ == "__main__":
    pattern = sys.argv[1]   #jobId
    caseDir = sys.argv[2]   #currentDir
    jobMan = "#SBATCH"
    if len(sys.argv) >= 4:
        jobMan = sys.argv[3]
    #currentDir内を探す
    files = findJobInCurrDir(caseDir, pattern)
    if len(files) == 0:
        #$HostDir以下を全て検索
        if caseDir.find("/./") >= 0:
            wdir = caseDir.split("/./")[0]
        else:
            #wdir = os.getenv("HOME")
            wdir = os.path.normpath(caseDir)
        files = findJobFile(wdir, pattern)
        caseDir = getCaseDir(files)
    logFile = getLogFile(caseDir, pattern, jobMan)
    print logFile

 
